@@ -61,7 +61,11 @@ def move_on_after(
 async def sleep_forever() -> NoReturn:
     """Pause execution of the current task forever (or until cancelled).
 
-    Equivalent to calling ``await sleep(math.inf)``.
+    Equivalent to calling ``await sleep(math.inf)``, except that if manually
+    rescheduled this will raise a `RuntimeError`.
+
+    Raises:
+      RuntimeError: if rescheduled
 
     """
     await trio.lowlevel.wait_task_rescheduled(lambda _: trio.lowlevel.Abort.SUCCEEDED)
@@ -186,7 +190,7 @@ def fail_after(
 # Users don't need to know that fail_at & fail_after wraps move_on_at and move_on_after
 # and there is no functional difference. So we replace the return value when generating
 # documentation.
-if "sphinx" in sys.modules:  # pragma: no cover
+if "sphinx.ext.autodoc" in sys.modules:
     import inspect
 
     for c in (fail_at, fail_after):
